# tim Rowledge tim at rowledge dot org
# ComponentScenes v0.3 15-06-12 -
# A plugin for SketchUp that is intended to produce a set of scenes that display the selected
# components individually, in a form suited to making dimensioned component views in LayOut.
#
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted, provided that the above
# copyright notice appear in all copies.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
# Changes
# attempt workaround for active_view.zoom failing to actually zoom to the chosen component

# To-Do's
# Make this an extension later http://sketchupapi.blogspot.ca/2011/07/turn-your-plugin-into-extension.html
# Package as an RBZ file http://sketchupapi.blogspot.ca/2011/12/rbzs-in-sketchup-8m2-distribute-your.html
# Do not make scenes for duplicate components 
# - (how to decide this? scaling, shearing and flipping in the component transform 
#	can make two apparently equivalent components NOT duplicates)
# Make undo/redo friendly
#	first pass at this is to wrap the main routine in start/commit_ operation
#	but it doesn't remove created pages and
#	SU crashes completely if it tries to undo a lot of component pages
# Find a way to close any open component edits after making the component pages so that
# the components don't appear to be missing; you could argue that the user should understand 
# but never underestimate the confusion of users
#	- on hold for now because it would surely annoy users to lose an edit deep inside a complex model


#-----------------------------------------------------------------------------

require 'sketchup.rb'

#-----------------------------------------------------------------------------
# set to true for debug output to console
#$VERBOSE = nil : sets 'Silent' mode 
#$VERBOSE = false : sets 'Medium' mode (default) 
#$VERBOSE = true : sets 'Verbose' mode 
$VERBOSE = false

module TimRowledge
module ComponentScenes

class ComponentScenesBuilder
# instance variables
	@selectedItems =  [] # a collection of the finally chosen components that need scenes

def self.buildSelectedItemsList(inputList)
# examine selection for duplicates, building a list of appropriate items. 
# Only components to be picked
	@selectedItems = []
	inputList.each {|comp|
	  if (comp.class == Sketchup::ComponentInstance) 
	    
	    # Checking for duplicate components in selection
	      dName = comp.definition.name
	      dup = false
	      
	      
	      @selectedItems.each {|item|
	        if (item.definition.name == dName)
	        	# more checks needed here
	        	# && item flip is  compatible with comp flip
	        	# && item scale is compatible with comp scale
				dup = true
	          	break
	        end  
	      }
	      
	      if dup == false
	      	@selectedItems.push(comp)
	      end
	  end
	  
	}

end #def buildSelectedItemsList

def self.makeComponentLayer(suggestedName)
# maker a new layer with a default unique name, make it hidden on all pages including future ones
    model = Sketchup.active_model
    layers = model.layers

	layerName = layers.unique_name(suggestedName)
	layer = layers.add layerName
    layer.page_behavior = LAYER_IS_HIDDEN_ON_NEW_PAGES
    model.pages.each { |pg| pg.set_visibility( layer, false ) }

	return layer
end #makeComponentLayer

def self.buildScenesForComponents()
# find the selected components, if any. quit if none; possible Dialogue to warn user
	theModel = Sketchup.active_model
	currentSelection = theModel.selection
	if currentSelection.empty?
		UI.messagebox("There is nothing currently selected.")
		return;
	end

# examine selection for duplicates, building a list of appropriate items. Only components to be picked
# if no suitable items, quit; possible Dialogue to warn user
	self.buildSelectedItemsList(currentSelection)
	if @selectedItems.empty?
		UI.messagebox("There are no suitable components currently selected.")
		return;
	end

	entitiesList = theModel.entities
	pageList = theModel.pages
# BUG WORKAROUND
# a fresh drawing has NO PAGES. So if  pageLIst is empty, add a first page named 'Scene1'
	if pageList.count == 0
		pageList.add("Scene1")
	end
# end workaround
	
	currentPage = pageList.selected_page

#	use a start/commit pair to wrap the operation. this speeds things 
#	up quite a bit and is suppose to make it more undo/redo friendly	
	theModel.start_operation("ComponentScenes", true)

	@selectedItems.each { |thisItem|
# 	for each item
#	create new page/scene & layer named for thisItem 
#		- may use definition, item name, possible extra for flipped or scaled
#	make everything hidden
#	make the new layer visible on the new page
#	create a copy of thisItem and place it on the new layer
# 	update the page to fix the visibility etc

#		derive a name; more details could be useful later - flip/scale status etc
	    partName = thisItem.definition.name.to_s
	    instName = thisItem.name.to_s
	    if instName != ""
	    	partName = partName + " (" + instName + ")"
	    end

#		adding a page moves us to using that new page as the active context. 
#		Despite the official doc, it returns the new page
  		page = pageList.add(partName)

#	hide all the items on the page; must be a better way to handle this
#	- perhaps some optimisation based on the way we copy the previous page each time after the first thisItem?
		entitiesList.each{|ent| ent.hidden= true}

#	make new hidden layer, visible only on this page
		layer = self.makeComponentLayer(partName)
		page.set_visibility(layer, true)

#		make a new component, set the transform to match the original,
#			add to the model's entity list,
#			then change the item's layer
		transf = Geom::Transformation.new [0,0,0]
		newItem = entitiesList.add_instance(thisItem.definition, transf)
		newItem.layer= layer
		
#		zoom into the component so there is a decnet chance we'll spot it
# BUG WORKAROUND
# attempt workaround for active_view.zoom failing to actually zoom to the chosen component
		currentSelection.clear
		currentSelection.add(newItem)
# end workaround
    	theModel.active_view.zoom(newItem)

#		update the page to fix the visibility etc
		page.update(127)
		
	}
# go back to the original page
	pageList.selected_page= currentPage
	theModel.commit_operation
	
end #def buildScenesForComponents()

end #class ComponentScenesBuilder

end #module ComponentScenes
end #module TimRowledge

if not file_loaded?(File.basename(__FILE__))
UI.menu("Plugins").add_item("Build Component Scenes") { TimRowledge::ComponentScenes::ComponentScenesBuilder.buildScenesForComponents() }
end

file_loaded(File.basename(__FILE__))   # Mark the script loaded.

